from _typeshed import StrOrBytesPath
from _typeshed.wsgi import StartResponse, WSGIApplication, WSGIEnvironment
from collections.abc import Iterable
from typing import IO, Any
from typing_extensions import Literal

from fanstatic.core import Library
from fanstatic.registry import LibraryRegistry
from webob import Request, Response
from webob.dec import wsgify
from webob.static import DirectoryApp, FileApp

MINUTE_IN_SECONDS: Literal[60]
HOUR_IN_SECONDS: Literal[3600]
DAY_IN_SECONDS: Literal[86400]
YEAR_IN_SECONDS: int
FOREVER: int

class BundleApp(FileApp):
    filenames: list[str]
    def __init__(self, rootpath: str, bundle: IO[bytes], filenames: Iterable[StrOrBytesPath]) -> None: ...
    @wsgify
    def __call__(self, req: Request) -> Response: ...

class LibraryPublisher(DirectoryApp):
    ignores: list[str]
    library: Library
    cached_apps: dict[str, FileApp]
    def __init__(self, library: Library) -> None: ...
    @wsgify
    def __call__(self, req: Request) -> Response: ...

class Publisher:
    registry: LibraryRegistry
    directory_publishers: dict[str, LibraryPublisher]
    def __init__(self, registry: LibraryRegistry) -> None: ...
    @wsgify
    def __call__(self, request: Request) -> Response: ...

class Delegator:
    app: WSGIApplication
    publisher: Publisher
    publisher_signature: str
    trigger: str
    def __init__(self, app: WSGIApplication, publisher: Publisher, publisher_signature: str = ...) -> None: ...
    def is_resource(self, request: Request) -> bool: ...
    def __call__(self, environ: WSGIEnvironment, start_response: StartResponse) -> Iterable[bytes]: ...

def make_publisher(global_config: Any) -> Publisher: ...



from abc import abstractmethod
from collections.abc import Iterable
from threading import Lock
from typing import Any, ClassVar, Protocol, TypeVar
from typing_extensions import Literal, Self

from fanstatic.compiler import Compiler, Minifier
from fanstatic.core import Library
from fanstatic.injector import InjectorPlugin
from pkg_resources import EntryPoint

class _HasName(Protocol):
    @property
    def name(self) -> str: ...

_NamedT = TypeVar("_NamedT", bound=_HasName)

prepare_lock: Lock

class Registry(dict[str, _NamedT]):
    @property
    @abstractmethod
    def ENTRY_POINT(self) -> str: ...
    def __init__(self, items: Iterable[_NamedT] = ()) -> None: ...  # noqaY011
    def add(self, item: _NamedT) -> None: ...
    def load_items_from_entry_points(self) -> None: ...
    def make_item_from_entry_point(self, entry_point: EntryPoint) -> Any: ...
    @classmethod
    def instance(cls) -> Self: ...

class LibraryRegistry(Registry[Library]):
    ENTRY_POINT: ClassVar[Literal["fanstatic.libraries"]]
    prepared: bool
    def prepare(self) -> None: ...

get_library_registry = LibraryRegistry.instance

class CompilerRegistry(Registry[Compiler]):
    ENTRY_POINT: ClassVar[Literal["fanstatic.compilers"]]

class MinifierRegistry(Registry[Minifier]):
    ENTRY_POINT: ClassVar[Literal["fanstatic.minifiers"]]

class InjectorRegistry(Registry[InjectorPlugin]):
    ENTRY_POINT: ClassVar[Literal["fanstatic.injectors"]]
